package graph;


import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Scanner;
import java.util.Set;
import java.util.TreeMap;
import java.util.zip.DataFormatException;



public class Atf 
{
	
	///////////////
	// CONSTANTS //
	///////////////

	private static final String FILE_EXT = ".atf";
	private static final String DELIMITERS = "\"|=";
	
	
	
	
	
	////////////
	// FIELDS //
	////////////
	
	private final Map<String,Attributes> attrsMap;
	private GraphElemType type;

	
	
	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////
	
	/**
	 * 
	 */
	private Atf()
	{
		this.attrsMap = new TreeMap<String,Attributes>();
	}
	
	/**
	 * 
	 */
	public Atf(GraphElemType type)
	{
		this();
		this.type = type;
	}
	
	/**
	 * 
	 * @param typeStr
	 */
	public Atf(String typeStr)
	{
		this(GraphElemType.parseType(typeStr));
	}
	
	/**
	 * 
	 * @param inputFile
	 * @throws IOException
	 * @throws DataFormatException 
	 */
	public Atf(File inputFile) throws IOException, DataFormatException
	{
		this();
		this.readFromFile(inputFile);
	}
	
	
	


	
	
	/////////////////////////
	// GETTERS AND SETTERS //
	/////////////////////////
	
	/**
	 * 
	 * @return
	 */
	public static String getFileExt()
	{
		return FILE_EXT;
	}
	
	/**
	 * 
	 * @return
	 */
	public GraphElemType getType()
	{
		return type;
	}
	
	/**
	 * 
	 * @return
	 */
	public Set<String> getElemIds()
	{
		return attrsMap.keySet();
	}
	
	/**
	 * 
	 * @param elemId
	 * @return
	 */
	public Attributes getAttrs(String elemId)
	{
		return attrsMap.get(elemId);
	}
	
	/**
	 * 
	 * @param elemId
	 * @param attrId
	 * @return
	 */
	public String getValue(String elemId, String attrId)
	{
		return getAttrs(elemId).getValue(attrId);
	}
	
	/**
	 * 
	 * @param elemId
	 * @return
	 */
	public Collection<String> getAttrIds(String elemId)
	{
		return attrsMap.get(elemId).getIds();
	}
	
	
	
	

	////////////////////
	// PUBLIC METHODS //
	////////////////////
	
	/**
	 * 
	 * @param elemId
	 * @param attrId
	 * @param attrVal
	 */
	public void add(String elemId, String attrId, String attrVal)
	{
		if(!attrsMap.containsKey(elemId))
			attrsMap.put(elemId, new Attributes());
		Attributes attrs = attrsMap.get(elemId);
		attrs.set(attrId, attrVal);
	}
	
	/**
	 * 
	 * @param elemId
	 * @param attrId
	 */
	public void remove(String elemId, String attrId)
	{
		if(attrsMap.containsKey(elemId))
		{
			Attributes attrs = attrsMap.get(elemId);
			attrs.remove(attrId);
		}
	}
	
	/**
	 * 
	 * @param elemId
	 */
	public void remove(String elemId)
	{
		attrsMap.remove(elemId);
	}
	
	/**
	 * 
	 * @param attrId
	 */
	public void removeAll(String attrId)
	{
		for(String elemId : attrsMap.keySet())
			remove(elemId, attrId);
	}
	
	/**
	 * 
	 * @param outputFile
	 * @throws IOException 
	 */
	public void writeToFile(File outputFile) throws IOException
	{
		BufferedWriter writer = null;
		try
		{
			writer = new BufferedWriter(new FileWriter(outputFile));
			writeToFile(writer);
		}
		finally
		{
			if(writer != null)
				writer.close();
		}
	}
	
	

	
	

	/////////////////////////////
	// PACKAGE PRIVATE METHODS //
	/////////////////////////////
	
	/**
	 * 
	 * @return
	 */
	Map<String,Attributes> getAttrsMap()
	{
		return attrsMap;
	}
	
	
	
	
	
	/////////////////////
	// PRIVATE METHODS //
	/////////////////////
	
	/**
	 * 
	 * @param inputFile
	 * @throws IOException
	 * @throws DataFormatException
	 */
	private void readFromFile(File inputFile) throws IOException, DataFormatException 
	{
		BufferedReader reader = null;
		try
		{
			reader = new BufferedReader(new FileReader(inputFile));
			readHeader(reader);
			String line;
			while((line = reader.readLine()) != null) 
			{
				parseLine(line);
			}
		}
		finally
		{
			if(reader != null)
				reader.close();
		}
	}
	
	/**
	 * 
	 * @param line
	 * @return
	 * @throws DataFormatException
	 * @throws IOException 
	 */
	private void readHeader(BufferedReader reader) throws IOException, DataFormatException
	{
		String line = reader.readLine();
		if(line == null)
			throw new DataFormatException("No AtfType specified");
		
		GraphElemType type = GraphElemType.parseType(line.trim());
		if(type == null)
			throw new DataFormatException("Invalid AtfType");
		
		this.type = type;
	}
	
	/**
	 * 
	 * @param line
	 * @throws DataFormatException 
	 */
	private void parseLine(String line) throws DataFormatException
	{
		line = line.trim();
		if(line.length() == 0) return;
		
		Scanner sc = new Scanner(line.replaceAll(DELIMITERS, " "));
		try
		{
			String elemId = sc.next();
			Attributes attrs = new Attributes();
			while(sc.hasNext())
			{
				String attrId = sc.next();
				String attrVal = sc.next();
				attrs.set(attrId, attrVal);
			}
			attrsMap.put(elemId, attrs);
		}
		catch(NoSuchElementException nsee)
		{
			throw new DataFormatException("Data format error in the line: " + line);
		}
	}
	
	/**
	 * 
	 * @param writer
	 * @throws IOException 
	 */
	private void writeToFile(BufferedWriter writer) throws IOException
	{
		writer.write(type.toString());
		writer.newLine();
		writer.newLine();
		for(String elemId : attrsMap.keySet())
		{
			Attributes attrs = attrsMap.get(elemId);
			writer.write(elemId + " ");
			for(String attrId : attrs.getIds())
			{
				String attrVal = attrs.getValue(attrId);
				writer.write("\"" + attrId + "\"=\"" + attrVal + "\" ");
			}
			writer.newLine();
			writer.flush();
		}
	}
	
}
