package graphview;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.InputMismatchException;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;
import java.util.Scanner;
import java.util.TreeMap;
import java.util.zip.DataFormatException;
import graph.Edge;
import graph.Edges;
import graph.Graph;
import graph.GraphSequence;
import graph.Vertex;



/**
 * 
 */
public class Mwadj 
{
	
	///////////////
	// CONSTANTS //
	///////////////
	
	private static final String FILE_EXT = ".mwadj";
	private static final String DELIMITERS = ":|,|;|\\("; 

	
	
	
	
	
	////////////
	// FIELDS //
	////////////
	
	private final GraphSequence graphSeq;
	
	
	
	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////
	
	/**
	 * 
	 * @param graphSeq
	 */
	public Mwadj(GraphSequence graphSeq)
	{
		this.graphSeq = graphSeq;
	}
	
	/**
	 * 
	 * @param inputFile
	 * @throws IOException 
	 * @throws DataFormatException 
	 * @throws InterruptedException 
	 */
	public Mwadj(File inputFile) throws IOException, DataFormatException
	{
		this.graphSeq = new GraphSequence();
		readFromFile(inputFile);
	}
	
	
	
	
	
	/////////////////////////
	// GETTERS AND SETTERS //
	/////////////////////////
	
	/**
	 * 
	 * @return
	 */
	public static String getFileExt()
	{
		return FILE_EXT;
	}
	
	/**
	 * 
	 * @return
	 */
	public GraphSequence getGraphSeq()
	{
		return this.graphSeq;
	}
	
	
	
	
	
	
	////////////////////
	// PUBLIC METHODS //
	////////////////////
	
	/**
	 * 
	 * @param outputFile
	 * @throws IOException
	 */
	public void writeToFile(File outputFile) throws IOException
	{
		BufferedWriter writer = null;
		try
		{
			writer = new BufferedWriter(new FileWriter(outputFile));
			MwadjMap mwadjMap = new MwadjMap(graphSeq);	
			for(String vert1Id : mwadjMap.keySet())
			{
				AdjList adjList = mwadjMap.get(vert1Id);
				writer.write(vert1Id + ": " + adjList + "\n");
				writer.flush();
			}
		}
		finally
		{
			if(writer != null)
				writer.close();
		}
	}
	
	
	
	
	
	
	/////////////////////
	// PRIVATE METHODS //
	/////////////////////
	
	/**
	 * 
	 * @param inputFile
	 * @throws IOException 
	 * @throws DataFormatException 
	 * @throws InterruptedException 
	 */
	private void readFromFile(File inputFile) throws IOException, DataFormatException
	{
		BufferedReader reader = null;
		try
		{
			reader = new BufferedReader(new FileReader(inputFile));
			String line = null;
			while((line = reader.readLine()) != null) 
			{
				parseLine(line);
			}
		}
		finally
		{
			if(reader != null)
				reader.close();
		}
	}
	
	/**
	 * 
	 * @param line
	 * @throws DataFormatException
	 */
	//TODO: Possibly make more elegant
	private void parseLine(String line) throws DataFormatException
	{
		line = line.trim();
		if(line.length() == 0) return;
		line = line.replaceAll(DELIMITERS, " ");
		line = line.replaceAll("\\)", " )");
		Scanner sc = new Scanner(line);
		try 
		{
			Vertex vert1 = new Vertex(sc.next());
			while(sc.hasNext())
			{
				Vertex vert2 = new Vertex(sc.next());
				String seqId = sc.next();
				while(!seqId.equals(")"))
				{
					graphSeq.addEdge(new Edge(vert1, vert2), seqId);
					seqId = sc.next();
				}	
			}
		}
		catch(InputMismatchException ime)
		{
			throw new DataFormatException("Data format error in the line: " + line);
		}
		catch(NoSuchElementException nsee)
		{
			throw new DataFormatException("Data format error in the line: " + line);
		}
	}
	






	////////////////////
	// NESTED CLASSES //
	////////////////////
	
	/**
	 * 
	 */
	@SuppressWarnings("serial")
	private class MwadjMap extends TreeMap<String,AdjList>
	{
		/**
		 * 
		 */
		private MwadjMap(GraphSequence graphSeq)
		{
			super();
			for(String seqId : graphSeq.getSeqIds())
			{
				Graph graph = graphSeq.getGraph(seqId);
				add(graph.getEdges(), seqId);
			}
		}
		
		/**
		 * 
		 * @param edges
		 */
		private void add(Edges edges, String seqId)
		{
			for(Edge edge : edges)
			{
				if(!this.containsKey(edge.getVert1Id()))
				{
					this.put(edge.getVert1Id(), new AdjList());
				}
				AdjList adjMap = this.get(edge.getVert1Id());
				adjMap.add(edge, seqId);
			}
		}
	}
	
	
	/**
	 * Provides a data structure to represent the information
	 * in a single line of an Mwadj file as an adjacency list.
	 */
	@SuppressWarnings("serial")
	private class AdjList extends TreeMap<String,List<SeqIdWeight>>
	{
		/**
		 * 
		 * @param edge
		 */
		private void add(Edge edge, String seqId)
		{
			if(!this.containsKey(edge.getVert2Id()))
			{
				List<SeqIdWeight> siwList = new ArrayList<SeqIdWeight>();
				this.put(edge.getVert2Id(), siwList);
			}
			List<SeqIdWeight> list = this.get(edge.getVert2Id());
			list.add(new SeqIdWeight(seqId, edge.getWeight()));
		}
		
		/**
		 * 
		 */
		@Override
		public String toString()
		{
			StringBuffer sb = new StringBuffer();
			for(String vert2Id : this.keySet())
			{
				sb.append(vert2Id);
				sb.append("(");
				List<SeqIdWeight> siwList = this.get(vert2Id);
				for(SeqIdWeight siw : siwList)
				{
					sb.append(siw.seqId + ",");
				}
				sb.deleteCharAt(sb.length()-1);
				sb.append("),");
			}
			sb.deleteCharAt(sb.length()-1);
			sb.append(';');
			return sb.toString();
		}
	}
	
	
	/**
	 * Stores a sequence number and edge weight as a pair of values.
	 */
	private class SeqIdWeight
	{
		private final String seqId;
		private final double weight;
		
		/**
		 * Creates a sequence number, edge weight pair.
		 * @param seqId
		 * @param weight
		 */
		private SeqIdWeight(String seqId, double weight)
		{
			this.seqId = seqId;
			this.weight = weight;
		}
	}
	
}
