/*
 * GraphTools
 */

package graphview;

import graph.Edge;
import graph.Edges;
import graph.Graph;
import graph.GraphFile;
import graph.Vertex;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Collection;
import java.util.InputMismatchException;
import java.util.NoSuchElementException;
import java.util.Scanner;
import java.util.zip.DataFormatException;




/**
 * Provides tools for working with the information related to a 
 * Graphview .wadj file.
 */
public class Wadj extends GraphFile
{
	
	///////////////
	// CONSTANTS //
	///////////////
	
	private static final String FILE_EXT = ".wadj";
	private static final String DELIMITERS = ":|\\(|\\)|,|;"; 
	
	
	
	
	
	//////////////////
	// CONSTRUCTORS //
	//////////////////
	
	/**
	 * Constructs a Wadj with the the specified Graph. Note that the
	 * graph is referenced, not copied, and so any changes to the graph 
	 * will be reflected in the Wadj.
	 * @param graph The graph to represent by this Wadj
	 */
	public Wadj(Graph graph)
	{
		super(graph);
	}
	
	/**
	 * Constructs a Wadj with the same information as the specified
	 * GraphFile. The underlying graph of the GraphFile is shared by
	 * this Wadj. 
	 * @param gf A GraphFile to be represented in the .wadj format.
	 */
	public Wadj(GraphFile gf)
	{
		super(gf);
	}
	
	/**
	 * Constructs a Wadj with the information in the specified File.
	 * If the File is not a properly formatted .wadj file an exception
	 * is thrown except in certain cases in which the formatting error
	 * can be ignored. Such cases should not be relied on for proper
	 * functioning.
	 * @param inputFile The .wadj file from which graph information is read
	 * @throws IOException
	 * @throws DataFormatException 
	 */
	public Wadj(File inputFile) throws IOException, DataFormatException
	{
		super(inputFile);
	}
	
	
	
	
	
	
	
	////////////////////
	// PUBLIC METHODS //
	////////////////////
	
	/**
	 * Gets the file extension for the wadj file type as a String
	 * which includes the '.' before the extension name.
	 */
	public static String getFileExt()
	{
		return FILE_EXT;
	}
	
	/**
	 * Reads graph data from the specified input file.
	 * @param inputFile The .wadj file from which graph information is read
	 * @throws IOException
	 * @throws DataFormatException 
	 */
	@Override
	public void readFromFile(File inputFile) throws IOException, DataFormatException 
	{
		BufferedReader reader = null;
		try
		{
			reader = new BufferedReader(new FileReader(inputFile));
			String line = null;
			while((line = reader.readLine()) != null) 
			{
				parseLine(line);
			}
		}
		finally
		{
			if(reader != null)
				reader.close();
		}
	}
	
	/**
	 * Writes underlying graph data to the specified File in the .wadj
	 * file format. 
	 * @param outputFile The file to write the graph information in the .wadj format
	 * @throws IOException 
	 */
	@Override
	public void writeToFile(File outputFile) throws IOException
	{
		BufferedWriter writer = null;
		try
		{
			writer = new BufferedWriter(new FileWriter(outputFile));
			writeFile(writer);
		}
		finally
		{
			if(writer != null)
				writer.close();
		}
	}
	
	
	
	
	
	/////////////////////
	// PRIVATE METHODS //
	/////////////////////
	
	/**
	 * Parses a single line of a .wadj file skipping blank lines and 
	 * whitespace.
	 * @param line A single line of a .wadj file
	 * @throws DataFormatException 
	 */
	private void parseLine(String line) throws DataFormatException
	{
		line = line.trim();
		if(line.length() == 0) return;
		Scanner sc = new Scanner(line.replaceAll(DELIMITERS, " "));
		try 
		{
			Vertex vert1 = new Vertex(sc.next());
			while(sc.hasNext())
			{
				Vertex vert2 = new Vertex(sc.next());
				Double weight = sc.nextDouble();
				graph.addEdge(new Edge(vert1, vert2, weight));
			}
		}
		catch(InputMismatchException ime)
		{
			throw new DataFormatException("Data format error in the line: " + line);
		}
		catch(NoSuchElementException nsee)
		{
			throw new DataFormatException("Data format error in the line: " + line);
		}
	}

	/**
	 * 
	 * @param writer
	 * @throws IOException 
	 */
	private void writeFile(BufferedWriter writer) throws IOException
	{
		Edges edges = graph.getEdges();
		for(String vert1Id : edges.getVert1Ids())
		{
			StringBuffer sb = new StringBuffer();
			sb.append(vert1Id + ":");
			Collection<Edge> incidentEdges = edges.getIncidentEdges(vert1Id);
			for(Edge edge : incidentEdges)
			{
				sb.append(edge.getVert2Id() + "(");
				sb.append(edge.getWeight() + "),");
			}
			sb.deleteCharAt(sb.length()-1);
			sb.append(";\n");
			writer.write(sb.toString());
			writer.flush();
		}
	}
	
}
